const ALPHABETS = [
    {
        name: 'base58',
        charset: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
    },
    {
        name: 'base64',
        charset: 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
    },
    {
        name: 'base64url',
        charset: 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_'
    }
];
function getAlphabet(name) {
    for (const alpha of ALPHABETS) {
        if (alpha.name === name) {
            return alpha.charset;
        }
    }
    throw TypeError('Charset does not exist: ' + name);
}
function encode$1(data, charset, padding = false) {
    const alphabet = getAlphabet(charset);
    const len = alphabet.length;
    const d = [];
    let s = '', i, j = 0, c, n;
    for (i = 0; i < data.length; i++) {
        j = 0;
        c = data[i];
        s += (c > 0 || (s.length ^ i) > 0) ? '' : '1';
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * 256 + c : c;
            c = n / len | 0;
            d[j] = n % len;
            j++;
        }
    }
    while (j-- > 0) {
        s += alphabet[d[j]];
    }
    return (padding && s.length % 4 > 0)
        ? s + '='.repeat(4 - s.length % 4)
        : s;
}
function decode$1(encoded, charset) {
    const alphabet = getAlphabet(charset);
    const len = alphabet.length, d = [], b = [];
    encoded = encoded.replace('=', '');
    let i, j = 0, c, n;
    for (i = 0; i < encoded.length; i++) {
        j = 0;
        c = alphabet.indexOf(encoded[i]);
        if (c < 0) {
            throw new Error(`Character range out of bounds: ${c}`);
        }
        i = (c > 0 || (b.length ^ i) > 0) ? i : b.push(0);
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * len + c : c;
            c = n >> 8;
            d[j] = n % 256;
            j++;
        }
    }
    while (j-- > 0) {
        b.push(d[j]);
    }
    return Uint8Array.from(b);
}
const BaseX = {
    encode: encode$1,
    decode: decode$1
};

const CHARSET = 'qpzry9x8gf2tvdw0s3jn54khce6mua7l';
const GENERATOR = [0x3b6a57b2, 0x26508e6d, 0x1ea119fa, 0x3d4233dd, 0x2a1462b3];
const encodings = {
    BECH32: 'bech32',
    BECH32M: 'bech32m'
};
function getEncodingConst(enc) {
    switch (enc) {
        case encodings.BECH32:
            return 1;
        case encodings.BECH32M:
            return 0x2bc830a3;
        default:
            throw new Error(`Unrecognized encoding: ${enc}`);
    }
}
function polymod(values) {
    let chk = 1;
    for (let p = 0; p < values.length; ++p) {
        const top = chk >> 25;
        chk = (chk & 0x1ffffff) << 5 ^ values[p];
        for (let i = 0; i < 5; ++i) {
            if (((top >> i) & 1) > 0) {
                chk ^= GENERATOR[i];
            }
        }
    }
    return chk;
}
function hrpExpand(hrp) {
    const ret = [];
    let p;
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) >> 5);
    }
    ret.push(0);
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) & 31);
    }
    return ret;
}
function convertBits(data, fromBits, toBits, pad = true) {
    const ret = [];
    let acc = 0;
    let bits = 0;
    const maxVal = (1 << toBits) - 1;
    const maxAcc = (1 << (fromBits + toBits - 1)) - 1;
    for (const val of data) {
        if (val < 0 || (val >> fromBits) > 0) {
            return [];
        }
        acc = ((acc << fromBits) | val) & maxAcc;
        bits += fromBits;
        while (bits >= toBits) {
            bits -= toBits;
            ret.push((acc >> bits) & maxVal);
        }
    }
    if (pad) {
        if (bits > 0) {
            ret.push((acc << (toBits - bits)) & maxVal);
        }
    }
    else if (bits >= fromBits || ((acc << (toBits - bits)) & maxVal) > 0) {
        return [];
    }
    return ret;
}
function verifyChecksum(hrp, data, enc) {
    const combined = hrpExpand(hrp).concat(data);
    return polymod(combined) === getEncodingConst(enc);
}
function createChecksum(hrp, data, enc) {
    const values = hrpExpand(hrp).concat(data).concat([0, 0, 0, 0, 0, 0]);
    const mod = polymod(values) ^ getEncodingConst(enc);
    const ret = [];
    for (let p = 0; p < 6; ++p) {
        ret.push((mod >> 5 * (5 - p)) & 31);
    }
    return ret;
}
function b32encode(hrp, data, enc) {
    const combined = data.concat(createChecksum(hrp, data, enc));
    let ret = hrp + '1';
    for (let p = 0; p < combined.length; ++p) {
        ret += CHARSET.charAt(combined[p]);
    }
    return ret;
}
function b32decode(bechstr, version) {
    const enc = (version > 0) ? 'bech32m' : 'bech32';
    if (!checkBounds(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    bechstr = bechstr.toLowerCase();
    if (!checkSeparatorPos(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    const data = [];
    const pos = bechstr.lastIndexOf('1');
    const hrp = bechstr.substring(0, pos);
    for (let p = pos + 1; p < bechstr.length; ++p) {
        const d = CHARSET.indexOf(bechstr.charAt(p));
        if (d === -1) {
            return { hrp: null, data: [0xFF] };
        }
        data.push(d);
    }
    return (verifyChecksum(hrp, data, enc))
        ? { hrp, data: data.slice(0, data.length - 6) }
        : { hrp: null, data: [0xFF] };
}
function checkBounds(bechstr) {
    let p;
    let char;
    let hasLower = false;
    let hasUpper = false;
    for (p = 0; p < bechstr.length; ++p) {
        char = bechstr.charCodeAt(p);
        if (char < 33 || char > 126) {
            return false;
        }
        if (char >= 97 && char <= 122) {
            hasLower = true;
        }
        if (char >= 65 && char <= 90) {
            hasUpper = true;
        }
    }
    return !(hasLower && hasUpper);
}
function checkSeparatorPos(bechstr) {
    const pos = bechstr.lastIndexOf('1');
    return !(pos < 1 ||
        pos + 7 > bechstr.length ||
        bechstr.length > 90);
}
function encode(data, hrp = 'bch', version = 0) {
    const dat = [version, ...convertBits([...data], 8, 5)];
    const enc = (version > 0) ? 'bech32m' : 'bech32';
    const str = b32encode(hrp, dat, enc);
    decode(str, version);
    return str;
}
function decode(str, version = 0) {
    const hrp = str.split('1', 1)[0];
    const { hrp: hrpgot, data } = b32decode(str, version);
    const decoded = convertBits(data.slice(1), 5, 8, false);
    const length = decoded.length;
    switch (true) {
        case (hrp !== hrpgot):
            throw new Error('Returned hrp string is invalid.');
        case (decoded === null || length < 2 || length > 40):
            throw new Error('Decoded string is invalid or out of spec.');
        case (data[0] > 16):
            throw new Error('Returned version bit is out of range.');
        case (data[0] === 0 && length !== 20 && length !== 32):
            throw new Error('Decoded string does not match version 0 spec.');
        case (data[0] === 0 && version !== 0):
            throw new Error('Decoded version bit does not match.');
        case (data[0] !== 0 && version !== 1):
            throw new Error('Decoded version bit does not match.');
        default:
            return Uint8Array.from(decoded);
    }
}
const Bech32 = {
    encode,
    decode
};

const ec = new TextEncoder();
const dc = new TextDecoder();
function strToBytes(str) {
    return ec.encode(str);
}
function hexToBytes(str) {
    const bytes = [];
    let i, idx = 0;
    if (str.length % 2 > 0) {
        throw new Error(`Invalid hex string length: ${str.length}`);
    }
    for (i = 0; i < str.length; i += 2) {
        bytes[idx] = parseInt(str.slice(i, i + 2), 16);
        idx += 1;
    }
    return Uint8Array.from(bytes);
}
function numToBytes(num) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xff;
        bytes.push(byte);
        num = (num - byte) / 256;
    }
    return Uint8Array.from(bytes);
}
function bigToBytes(big) {
    const bytes = [];
    while (big > 0n) {
        const byte = big & 0xffn;
        bytes.push(Number(byte));
        big = (big - byte) / 256n;
    }
    return Uint8Array.from(bytes);
}
function bytesToStr(bytes) {
    return dc.decode(bytes);
}
function bytesToHex(bytes) {
    const hex = [];
    let i;
    for (i = 0; i < bytes.length; i++) {
        hex.push(bytes[i].toString(16).padStart(2, '0'));
    }
    return hex.join('');
}
function bytesToNum(bytes) {
    let num = 0, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256) + bytes[i];
    }
    return Number(num);
}
function bytesToBig(bytes) {
    let num = 0n, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256n) + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function buffer(value) {
    if (value instanceof ArrayBuffer) {
        return new Uint8Array(value);
    }
    if (value instanceof Uint8Array) {
        return value;
    }
    const type = typeof value;
    switch (type) {
        case 'bigint':
            return bigToBytes(value);
        case 'boolean':
            return Uint8Array.of(value);
        case 'number':
            return numToBytes(value);
        case 'string':
            return (value.match(/^(02|03)*[0-9a-fA-F]{64}$/) !== null)
                ? hexToBytes(value)
                : ec.encode(value);
        default:
            throw TypeError('Unsupported format:' + type);
    }
}

async function sha256(rawdata) {
    return crypto.subtle.digest('SHA-256', rawdata)
        .then(buff => new Uint8Array(buff));
}
async function addChecksum(data) {
    const sum = await sha256(data);
    return Uint8Array.of(...data, ...sum.slice(0, 4));
}
async function checkTheSum(data) {
    const dat = data.slice(0, -4);
    const sum = (await sha256(dat)).slice(0, 4);
    if (sum.toString() !== dat.toString()) {
        throw new Error('Invalid checksum!');
    }
    return data;
}

const { crypto: crypto$1 } = globalThis;
const Hex = {
    encode: (x) => Buff.raw(x).toHex(),
    decode: (x) => Buff.hex(x).toBytes(),
    normalize: (x) => Buff.normalize(x)
};
const Txt = {
    encode: (x) => Buff.raw(x).toStr(),
    decode: (x) => Buff.str(x).toBytes(),
    serialzie: (x) => Buff.serialize(x),
    revitalize: (x) => Buff.revitalize(x)
};
const Base64 = {
    encode: (x) => BaseX.encode(x, 'base64'),
    decode: (x) => BaseX.decode(x, 'base64'),
    encodeUrl: (x) => BaseX.encode(x, 'base64'),
    decodeUrl: (x) => BaseX.decode(x, 'base64')
};
class Buff extends Uint8Array {
    static { this.num = (number, size, orient = 'le') => {
        return new Buff(numToBytes(number), size, orient);
    }; }
    static { this.big = (number, size, orient = 'le') => {
        return new Buff(bigToBytes(number), size, orient);
    }; }
    static async b58check(x) {
        const decoded = BaseX.decode(x, 'base58');
        return new Buff(await checkTheSum(decoded));
    }
    static { this.buff = (x, s) => new Buff(buffer(x), s); }
    static { this.raw = (x, s) => new Buff(x, s); }
    static { this.str = (x, s) => new Buff(strToBytes(x), s); }
    static { this.hex = (x, s) => new Buff(hexToBytes(x), s); }
    static { this.json = (x) => new Buff(strToBytes(JSON.stringify(x))); }
    static { this.base64 = (x) => new Buff(BaseX.decode(x, 'base64')); }
    static { this.b64url = (x) => new Buff(BaseX.decode(x, 'base64url')); }
    static { this.bech32 = (x, ver) => new Buff(Bech32.decode(x, ver)); }
    constructor(data, size = null, orient = 'be') {
        if (size !== null) {
            const tmp = new Uint8Array(size).fill(0);
            tmp.set(new Uint8Array(data));
            data = tmp.buffer;
        }
        data = (orient === 'le')
            ? new Uint8Array(data).reverse()
            : data;
        super(data);
        return this;
    }
    get num() {
        return this.toNum();
    }
    get big() {
        return this.toBig();
    }
    get arr() {
        return this.toArr();
    }
    get str() {
        return this.toStr();
    }
    get hex() {
        return this.toHex();
    }
    get raw() {
        return new Uint8Array(this);
    }
    get hash() {
        return this.toHash();
    }
    get id() {
        return this.toHash().then(raw => new Buff(raw).hex);
    }
    toNum(orient = 'le') {
        return (orient === 'le')
            ? bytesToNum(this.reverse())
            : bytesToNum(this);
    }
    toBig(orient = 'le') {
        return (orient === 'le')
            ? bytesToBig(this.reverse())
            : bytesToBig(this);
    }
    async toHash() {
        return crypto$1.subtle.digest('SHA-256', this.raw)
            .then(buff => new Uint8Array(buff));
    }
    async toBase58() {
        return BaseX.encode(await addChecksum(this), 'base58');
    }
    toArr() { return Array.from(this); }
    toStr() { return bytesToStr(this); }
    toHex() { return bytesToHex(this); }
    toJson() { return JSON.parse(bytesToStr(this)); }
    toBytes() { return new Uint8Array(this); }
    toB64url() { return BaseX.encode(this, 'base64url'); }
    toBase64(padding) { return BaseX.encode(this, 'base64', padding); }
    toBech32(hrp, ver) { return Bech32.encode(this, hrp, ver); }
    prepend(data) {
        return Buff.of(...data, ...this);
    }
    append(data) {
        return Buff.of(...this, ...data);
    }
    slice(start, end) {
        return new Buff(new Uint8Array(this).slice(start, end));
    }
    reverse() {
        return new Buff(new Uint8Array(this).reverse());
    }
    write(bytes, offset) {
        this.set(bytes, offset);
    }
    prependVarint(num = this.length) {
        return Buff.of(...Buff.readVarint(num), ...this);
    }
    static from(data) {
        return new Buff(Uint8Array.from(data));
    }
    static of(...args) {
        return new Buff(Uint8Array.of(...args));
    }
    static join(arr) {
        let i, idx = 0;
        const totalSize = arr.reduce((prev, curr) => prev + curr.length, 0);
        const totalBytes = new Uint8Array(totalSize);
        for (const bytes of arr) {
            for (i = 0; i < bytes.length; idx++, i++) {
                totalBytes[idx] = bytes[i];
            }
        }
        return new Buff(totalBytes, totalSize);
    }
    static readVarint(num) {
        if (num < 0xFD) {
            return Buff.num(num, 1);
        }
        else if (num < 0x10000) {
            return Buff.of(0xFD, ...Buff.num(num, 2));
        }
        else if (num < 0x100000000) {
            return Buff.of(0xFE, ...Buff.num(num, 4));
        }
        else if (num < 0x10000000000000000) {
            return Buff.of(0xFF, ...Buff.num(num, 8));
        }
        else {
            throw new Error(`Value is too large: ${num}`);
        }
    }
    static random(size = 32) {
        return new Buff(crypto$1.getRandomValues(new Uint8Array(size)));
    }
    static { this.encode = strToBytes; }
    static { this.decode = bytesToStr; }
    static normalize(data, size) {
        if (data instanceof Uint8Array)
            return data;
        if (typeof data === 'string')
            return Buff.hex(data, size).toBytes();
        if (typeof data === 'number')
            return Buff.num(data, size).toBytes();
        if (typeof data === 'bigint')
            return Buff.big(data, size).toBytes();
        throw TypeError(`Unrecognized format: ${typeof data}`);
    }
    static serialize(data) {
        if (typeof data === 'string') {
            return Buff.str(data).toBytes();
        }
        if (typeof data === 'object') {
            if (data instanceof Uint8Array) {
                return data;
            }
            try {
                return Buff.json(data).toBytes();
            }
            catch {
                throw TypeError('Object is not serializable.');
            }
        }
        throw TypeError(`Unrecognized format: ${typeof data}`);
    }
    static revitalize(data) {
        if (data instanceof Uint8Array) {
            data = bytesToStr(data);
        }
        if (typeof data === 'string') {
            try {
                return JSON.parse(data);
            }
            catch {
                return data;
            }
        }
        return data;
    }
}

const is = {
    null: x => x === null,
    undefined: x => typeof x === 'undefined',
    hex: x => isHex(x),
    string: x => typeof x === 'string',
    infinity: x => x === Infinity,
    bigint: x => typeof x === 'bigint',
    number: x => typeof x === 'number',
    class: x => (typeof x?.prototype === 'object' &&
        x.toString().startsWith('class')),
    function: x => typeof x === 'function',
    uint8: x => x instanceof Uint8Array,
    uint16: x => x instanceof Uint16Array,
    uint32: x => x instanceof Uint32Array,
    buffer: x => x instanceof ArrayBuffer,
    array: x => Array.isArray(x),
    object: x => typeof x === 'object'
};
const array = {
    isString: x => x.every((e) => is.string(e)),
    isNumber: x => x.every((e) => is.number(e)),
    isBigint: x => x.every((e) => is.bigint(e))
};
const type = (x) => {
    for (const [k, v] of Object.entries(is)) {
        if (v(x) === true) {
            return k;
        }
    }
    return 'unknown';
};
function isHex(str) {
    switch (true) {
        case (typeof str !== 'string'):
            return false;
        case (/[^0-9a-fA-F]/.test(str)):
            return false;
        case (str.length % 2 !== 0):
            return false;
        default:
            return true;
    }
}
const Check = {
    type,
    array,
    is
};

class Stream {
    constructor(data) {
        this.data = new Uint8Array(data);
        this.size = this.data.length;
    }
    peek(size) {
        if (size > this.size) {
            throw new Error(`Size greater than stream: ${size} > ${this.size}`);
        }
        return new Buff(this.data.slice(0, size).buffer);
    }
    read(size) {
        size = size ?? this.readVarint();
        const chunk = this.peek(size);
        this.data = this.data.slice(size);
        this.size = this.data.length;
        return chunk;
    }
    readVarint() {
        const num = this.read(1).toNum();
        switch (true) {
            case (num >= 0 && num < 0xFD):
                return num;
            case (num === 0xFD):
                return this.read(2).toNum();
            case (num === 0xFE):
                return this.read(4).toNum();
            case (num === 0xFF):
                return this.read(8).toNum();
            default:
                throw new Error(`Varint is out of range: ${num}`);
        }
    }
}

export { Base64, BaseX, Bech32, Buff, Check, Hex, Stream, Txt };
//# sourceMappingURL=module.js.map
